// Picassobot - Robotic Drawing Arm kit
// Copyright (c) 2014 - The Educated Robot

class Arm
{
    // Variables for the arm with initial settings. (Measurements in millimeters * 2 = pixels)
    float shoulderX = (265+10)*2;      // Location of the shoulder pivut point relative to the top left of the screen.
    float shoulderY = (100+40)*2;
    float upperArmLength = 124*2;      // Length of arms (to drawing pen point)
    float lowerArmLength = 136*2;

    float shoulderAngle = 0;
    float shoulderAnglemin = radians(45);
    float shoulderAnglemax = radians(198);
    float elbowX = 700;
    float elbowY = 250;
    float elbowAngle = 0;
    float elbowAnglemin=radians(0);
    float elbowAnglemax=radians(160);
    float wristX = 686; // Relative to shoulder pivut point.
    float wristY = 157;
    float targetX = 686; // Relative to shoulder pivut point. (always -x)
    float targetY = 157;
    int servo1 = 0;
    int servo2 = 0;
    boolean wristUp = false;
    boolean limitServos=true;
    float armLengthDifference;
    boolean wristDown = false;
  
    Arm()
    {
        armLengthDifference = abs(upperArmLength-lowerArmLength);
    }
    
    void drawBone(float x1, float y1, float x2, float y2, int drawColor)
    {
        fill(0);
        stroke(drawColor);
        line(x1, y1, x2, y2);
    }
  
    void drawJoint(float x1, float y1, int drawColor)
    {
        fill(drawColor, 64);
        stroke(drawColor);
        ellipse(x1, y1, 12, 12);
    }
  
    void update()
    {
        
        // Calculate the arm positions and angles.
        calculateAngles();
  
        // Draw the reach extants of arms limited by servo angles. NOTE: First four lines show full arc, second are limited to board.
        stroke(#FFCC00); 
        noFill();
        //stroke(#000000); 
        //ellipse(shoulderX,shoulderY,2*(upperArmLength+lowerArmLength),2*(upperArmLength+lowerArmLength));
        //ellipse(shoulderX,shoulderY,2*(sqrt(sq(upperArmLength)+sq(lowerArmLength)-2*upperArmLength*lowerArmLength*cos(PI-elbowAnglemin))),2*(sqrt(sq(upperArmLength)+sq(lowerArmLength)-2*upperArmLength*lowerArmLength*cos(PI-elbowAnglemin))));
        //arc(cos(shoulderAnglemin)*upperArmLength+shoulderX,sin(shoulderAnglemin)*upperArmLength+shoulderY,2*lowerArmLength,2*lowerArmLength,0+shoulderAnglemin,PI+shoulderAnglemin);
        //arc(cos(shoulderAnglemax)*upperArmLength+shoulderX,sin(shoulderAnglemax)*upperArmLength+shoulderY,2*lowerArmLength,2*lowerArmLength,0+shoulderAnglemax,PI+shoulderAnglemax);
        if (false) {
          arc(shoulderX, shoulderY, 2*(upperArmLength+lowerArmLength), 2*(upperArmLength+lowerArmLength), 2.71, 3.53);
          arc(shoulderX,shoulderY,2*(sqrt(sq(upperArmLength)+sq(lowerArmLength)-2*upperArmLength*lowerArmLength*cos(PI-elbowAnglemax))),2*(sqrt(sq(upperArmLength)+sq(lowerArmLength)-2*upperArmLength*lowerArmLength*cos(PI-elbowAnglemax))), 2.35, 5.05);
          arc(cos(shoulderAnglemin)*upperArmLength+shoulderX,sin(shoulderAnglemin)*upperArmLength+shoulderY,2*lowerArmLength,2*lowerArmLength,2.99,3.58);
          arc(cos(shoulderAnglemax)*upperArmLength+shoulderX,sin(shoulderAnglemax)*upperArmLength+shoulderY,2*lowerArmLength,2*lowerArmLength,5.82,6.25);
        }
        
        // Draw the arm itself.
        drawBone(shoulderX, shoulderY, elbowX, elbowY, #FF0000); //#005FFF); // Upper arm
        drawJoint(shoulderX, shoulderY, #005FFF); // Shoulder
        drawBone(elbowX, elbowY, wristX, wristY, #FF0000); // Lower arm
        drawJoint(elbowX, elbowY, #FF0000); // Elbow
        drawJoint(wristX, wristY, #00FF00); // Wrist
        
        // Draw upper arm outline.
        pushMatrix();
        translate(shoulderX, shoulderY);
        rotate(shoulderAngle);
        scale(upperArmLength);
        fill(#0000FF, 100);
        noStroke();
        arc(0, 0, .26, .26, HALF_PI, PI+HALF_PI);
        rect(0, -.13, .23, .26);
        triangle(.23, -.13, .23, -.08, .28, -.08);
        triangle(.23, .13, .23, .08, .28, .08);
        rect(.23, -.08, .44, .16);
        triangle(.67, -.13, .67, -.08, .62, -.08);
        triangle(.67, .13, .67, .08, .62, .08);
        rect(.67, -.13, .33, .26);
        arc(1, 0, .26, .26, -HALF_PI, HALF_PI);
        noTint();
        popMatrix();
        
        // Draw lower arm outline.
        pushMatrix();
        translate(elbowX, elbowY);
        rotate(elbowAngle+shoulderAngle);
        scale(lowerArmLength);
        fill(#0000FF, 100);
        noStroke();
        rect(-.09, -.08, .96, .16, .04);
        rect(.77, -.04, .22, .02);
        rect(.77, .02, .22, .02);
        rect(.94, -.10, .02, .20);
        noTint();
        popMatrix();
    }
  
    private void calculateAngles()
    {
        float distanceFromShoulder;
        float angleFromShoulder;
        
        // Start by calculating the distance and angle from the shoulder to the target (mouse).
        if (targetX > -armLengthDifference/2)
        {
            //targetX = -armLengthDifference/2;
        }
        angleFromShoulder = atan2(targetY, targetX);
        distanceFromShoulder = sqrt(sq(targetX)+sq(targetY));
  
        // Check to see if target (mouse) is able to be reached.
        if (distanceFromShoulder > upperArmLength+lowerArmLength)
        {
            // Mouse is too far for arm to reach.
            wristX = cos(angleFromShoulder)*(upperArmLength+lowerArmLength);
            wristY = sin(angleFromShoulder)*(upperArmLength+lowerArmLength);
        }
        else if (distanceFromShoulder < armLengthDifference)
        {
            // Mouse is to close to shoulder to reach.
            wristX = cos(angleFromShoulder)*(armLengthDifference+.1);
            wristY = sin(angleFromShoulder)*(armLengthDifference+.1);
        }
        else {
            // Mouse is within drawing area.
            //wristX = round(targetX);
            //wristY = round(targetY);
            wristX = targetX;
            wristY = targetY;
        }

//          cursor(CROSS);
//print(wristX + " : " + wristY + " - " + shoulderX + " : " + shoulderY + "\n");
        if(limitServos) {
            // Check if wrist is in drawing area with borders.
            if (wristX < -470)
                wristX = -470;
            if (wristX > -116)
                wristX = -116;
            if (wristY < -96)
                wristY = -96;
            if (wristY > 180)
                wristY = 180;
        }

        // Recalculate distance from shoulder based on new wrist position.
        angleFromShoulder = atan2(wristY, wristX);
        distanceFromShoulder = sqrt(sq(wristX)+sq(wristY));
  
        shoulderAngle = angleFromShoulder - acos((sq(upperArmLength)+sq(wristX)+sq(wristY)-sq(lowerArmLength))/(2*upperArmLength*distanceFromShoulder) - 0.0000001);
        if (shoulderAngle < 0)
            shoulderAngle += TWO_PI;
    
        if (limitServos) {
            if(shoulderAngle<shoulderAnglemin)
                shoulderAngle=shoulderAnglemin;
            else if(shoulderAngle>shoulderAnglemax)
                shoulderAngle=shoulderAnglemax;
        }
    
        elbowAngle = atan2((wristY-upperArmLength*sin(shoulderAngle))/lowerArmLength,(wristX-upperArmLength*cos(shoulderAngle))/lowerArmLength)-shoulderAngle;
        if (elbowAngle < 0)
            elbowAngle += TWO_PI;
    
        if(limitServos) {
            if(elbowAngle>elbowAnglemax)
                elbowAngle=elbowAnglemax;
            else if(elbowAngle<elbowAnglemin)
                elbowAngle=elbowAnglemin;
        }
  
        elbowY = sin(shoulderAngle)*upperArmLength+shoulderY;
        elbowX = cos(shoulderAngle)*upperArmLength+shoulderX;
    
        wristY = sin(elbowAngle+shoulderAngle)*lowerArmLength+elbowY;
        wristX = cos(elbowAngle+shoulderAngle)*lowerArmLength+elbowX;
        
        //servo1 = int(map(arm.shoulderAngle, 0, TWO_PI, 600, 2400));
        //servo2 = int(map(arm.elbowAngle, 0, TWO_PI, 600, 2400));
        //servo1 = int(map(degrees(arm.shoulderAngle), 52, 168, 600, 2400));
        //servo2 = int(map(degrees(arm.elbowAngle), 30, 154, 600, 2400));
        servo1 = int(map(degrees(arm.shoulderAngle), g_servoAngles[0], g_servoAngles[1], g_servoPulses[0], g_servoPulses[1]));
        servo2 = int(map(degrees(arm.elbowAngle), g_servoAngles[2], g_servoAngles[3], g_servoPulses[2], g_servoPulses[3]));
if (output != null)
  output.print("Wrist "+wristX + " : " + wristY + " Shoulder " + shoulderX + " : " + shoulderY + " Servo " + servo1 + " : " + servo2+ "\n");
    }
  
    void setTargetXY(float newX, float newY)
    {
        if (Float.isNaN(newX) || Float.isNaN(newY)) {
          println("NaN");
         return;
        }
        targetX = newX - shoulderX;
        targetY = newY - shoulderY;  
    }
    
    void setWristDown(boolean down)
    {
        wristDown = down;
    }
  
    void toggleLimits()
    {
        limitServos=!limitServos;
    }
}













