// Picassobot - Robotic Drawing Arm kit
// Copyright (c) 2014 - The Educated Robot

class ScrollBar {
  String txt;
  int x, y; // The x- and y-coordinates
  float sw, sh; // Width and height of scrollbar
  float pos; // Position of thumb
  float posMin, posMax; // Max and min values of thumb
  boolean rollover; // True when the mouse is over
  boolean locked; // True when its the active scrollbar
  float minVal, maxVal; // Min and max values for the thumb
  int dp; // Number of digits to the right of decimal to show (default is 0) 
  boolean active; // Is it visible?
  
  ScrollBar(String t, int xp, int yp, int w, int h, float miv, float mav, float initialPos, int decimalPlace) {
    txt = t;
    x = xp;
    y = yp;
    sw = w;
    sh = h;
    minVal = miv;
    maxVal = mav;
    //pos = x + sw / 2 - sh / 2;
    setPos(initialPos);
    posMin = x;
    posMax = x + sw - sh;
    dp = decimalPlace;
    active = true;
  }
 
  // Updates the over boolean and the position of the thumb
  void update(int mx, int my) {
    if (over(mx, my) == true) {
      rollover = true;
    } else {
      rollover = false;
    }
    if (locked == true) {
      pos = constrain(mx - sh / 2, posMin, posMax);
    }
  }
 
  // Locks the thumb so the mouse can move off and still update
  void press() {
    if (rollover == true) {
      locked = true;
    } else {
      locked = false;
    }
  }
 
  // Resets the scrollbar to neutral
  void release() {
    locked = false;
  }
   
  // Returns true if the cursor is over the scrollbar
  boolean over(int mx, int my) {
    if (active && (mx > x) && (mx < x + sw) && (my > y) && (my < y + sh)) {
      return true;
    } else {
      return false;
    }
  }
 
  void setActive(boolean _active)
  {
    active = _active;
  }
  
  // Draws the scrollbar to the screen
  void display() {
    if (active) {
      stroke(0);
      fill(0);
      textAlign(LEFT, BOTTOM);
      text(txt, x, y);
      textAlign(RIGHT, BOTTOM);
      if (dp == 0)
        text(round(getPos()), x+sw, y);
      else
        text(nf(getPos(), 0, dp), x+sw, y);
      fill(255);
      rect(x, y, sw, sh, 8);
      if ((rollover == true) || (locked == true)) {
        fill(0);
      } else {
        fill(102);
      }
      rect(pos, y+1, sh, sh-2, 8);
    }
  }
  
  // Returns the current value of the thumb
  float getPos() {
    float scalar = sw / (sw - sh);
    float ratio = (pos - x) * scalar;
    float offset = minVal + (ratio / sw * (maxVal - minVal));
    return offset;
  }
  
  // Sets the current value of the thumb
  void setPos(float offset) {
    float scalar = sw / (sw - sh);
    pos = ((offset - minVal) * sw / (maxVal - minVal)) / scalar + x;
  }
}
