// Picassobot - Robotic Drawing Arm kit
// Copyright (c) 2014 - The Educated Robot

int calibrationState = 0;
String calibrationText = "";

void doNextCalibrationState()
{
    switch(calibrationState)
    {
        //arm.setTargetXY(185, 272);
      case 0:
        // Move to calibrate based on bottom left value.
        calibrationText = "Move the servo values below so that the pen tip is \ndirectly above the bottom LEFT corner of the \ndrawing area." +
                          "\n\nAdjust the wrist servo so that the pen tip is lifted \nabout 2mm above the paper." +
                          "\n\nClick the Next button to continue.";
        arm.setTargetXY(80, 460);
        scrollBar[SBAR_SERVO1].setPos(1888); // Shoulder servo "reset" setting.
        scrollBar[SBAR_SERVO2].setPos(1430); // Elbow servo "reset" setting.
        scrollBar[SBAR_SERVO3].setPos(1430); // Wrist servo "reset" setting.
        break;
      
      case 1:
         // Save out calibration values for lower left corner.
        g_servoPulses[0] = round(scrollBar[SBAR_SERVO1].getPos());
        g_servoPulses[2] = round(scrollBar[SBAR_SERVO2].getPos());
        g_servoPulses[5] = round(scrollBar[SBAR_SERVO3].getPos());
        savePulses();
        
        calibrationText = "Move the servo values below so that the pen tip is \ndirectly above the bottom RIGHT corner of the \ndrawing area.";
        arm.setTargetXY(434, 460);
        break;
      case 2:
        // Save out calibration values for lower right corner.
        g_servoPulses[1] = round(scrollBar[SBAR_SERVO1].getPos());
        g_servoPulses[2] = round(scrollBar[SBAR_SERVO2].getPos());
        g_servoPulses[5] = round(scrollBar[SBAR_SERVO3].getPos());
        savePulses();

        // Return to main page.        
        calibrationText = "";
        curHead = curNode = null;
        setDefaultAction();
        button[BUTTON_STOP].setSelected(true);
        key = '/';
        keyPressed();
        break;
    }
    ++calibrationState;
}

void drawCalibration()
{
      drawBoard();
      arm.update();
      button[BUTTON_CALIBRATENEXT].draw();
      button[BUTTON_CALIBRATECANCEL].draw();
      for (int i=SBAR_SERVO1; i <= SBAR_SERVO3; i++) {
        scrollBar[i].update(mouseX, mouseY);
        scrollBar[i].display();
      }
      textAlign(LEFT, TOP);
      fill(#FF0000);
      //fill(0);
      text(calibrationText, 100, 200);

      drawStatus();
      sendCalibrationCommand();
      // Draw codes being transmitted.
        noStroke();
        fill(70, 200);
        rect(width-2, height-22, -250, -37);
        textAlign(CENTER, CENTER);
        if (launchPad == null)
          fill(0, 200, 0);
        else {
          fill(50, 0, 0);
          text("Port: " + launchPadPort, width-126, height-50);
        }
        text(lastCommandSent, width-126, height-32);

}

void sendCalibrationCommand()
{
      String servo1 = "0000" + round(scrollBar[SBAR_SERVO1].getPos()); // Shoulder
      String servo2 = "0000" + round(scrollBar[SBAR_SERVO2].getPos()); // Elbow
      String servo3 = "0000" + round(scrollBar[SBAR_SERVO3].getPos()); // Wrist
      lastCommandSent = "W" + servo3.substring(servo3.length() - 4) + "E" + servo2.substring(servo2.length() - 4) + "S" + servo1.substring(servo1.length() - 4) + "\r";
    try
    {
      launchPad.write(lastCommandSent);
      delay(100);
    } catch (Exception e)
    {
      launchPad = null;
      retrySerial = 200;
      //delay(10000);
      //connectSerial();
    }

}

